<?php

namespace WPForms\Integrations\PayPalCommerce\Process\Exceptions;

/**
 * Exception thrown when a required shipping parameter is missing.
 *
 * @since 1.10.0
 */
class MissingRequiredShippingParam extends \Exception {
}
