<?php

namespace WPForms\Emails\Templates;

/**
 * Class Classic.
 * This is an updated version of our standard email template.
 *
 * @since 1.8.5
 */
class Classic extends Notifications {

	/**
	 * Template slug.
	 *
	 * @since 1.8.5
	 *
	 * @var string
	 */
	const TEMPLATE_SLUG = 'classic';
}
