<?php

namespace WPForms\Emails\Templates;

/**
 * Class Compact.
 * As the name suggests, it's a compact variant of the Classic template.
 *
 * @since 1.8.5
 */
class Compact extends Notifications {

	/**
	 * Template slug.
	 *
	 * @since 1.8.5
	 *
	 * @var string
	 */
	const TEMPLATE_SLUG = 'compact';
}
